@@ -9,31 +9,42 @@ if [[ $(uname) == Darwin ]]; then
   # export LLVM_CONFIG explicitly as the one installed from llvmdev
   # in the build root env, the one in the bootstrap location needs to be ignored.
   export LLVM_CONFIG=$(ls $(which llvm-config))
-  # now bootstrap the toolchain for building
+  
+  # Set conda subdir and bootstrap with x86_64 compiler
+  CONDA_SUBDIR=osx-64
   conda create -y -p ${PWD}/bootstrap clangxx_osx-64
+  
   SRC_DIR=${PWD}
   export PATH=${SRC_DIR}/bootstrap/bin:${PATH}
   CONDA_PREFIX=${SRC_DIR}/bootstrap \
     . ${SRC_DIR}/bootstrap/etc/conda/activate.d/*
-  SDKPATH=$(xcrun --show-sdk-path)
+  
+  # Use explicit SDK path if set, otherwise detect
+  if [ -z "$SDKROOT" ]; then
+    SDKPATH=$(xcrun --show-sdk-path)
+  else
+    SDKPATH=$SDKROOT
+  fi
   export CONDA_BUILD_SYSROOT=${CONDA_BUILD_SYSROOT:-${SDKPATH}}
+  
+  # Set minimum deployment target if not already set
+  if [ -z "$MACOSX_DEPLOYMENT_TARGET" ]; then
+    export MACOSX_DEPLOYMENT_TARGET=11.0
+  fi
+  
   export CXXFLAGS=${CFLAGS}" -mmacosx-version-min=${MACOSX_DEPLOYMENT_TARGET}"
   export CFLAGS=${CFLAGS}" -mmacosx-version-min=${MACOSX_DEPLOYMENT_TARGET}"
   SYSROOT_DIR=${CONDA_BUILD_SYSROOT}
   CFLAG_SYSROOT="--sysroot ${SYSROOT_DIR}"
   ${LLVM_CONFIG} --version
   export SDKROOT=${SDKPATH}
-fi
 
-if [ -n "$MACOSX_DEPLOYMENT_TARGET" ]; then
-    # OSX needs 10.7 or above with libc++ enabled
-    export MACOSX_DEPLOYMENT_TARGET=10.9
-fi
-
-if [[ ${MACOSX_DEPLOYMENT_TARGET} == 10.9 ]]; then
   DARWIN_TARGET=x86_64-apple-darwin13.4.0
 fi
 
+if [ -n "$MACOSX_DEPLOYMENT_TARGET" ]; then
+    export MACOSX_DEPLOYMENT_TARGET
+fi
 
 # Make sure any error below is reported as such
 set -v -e
