@@ -11,11 +11,7 @@ source deactivate
 set -v
 # Display root environment (for debugging)
 conda list
-# Clean up any left-over from a previous build
-# (note workaround for https://github.com/conda/conda/issues/2679:
-#  `conda env remove` issue)
-conda remove --all -q -y -n $CONDA_ENV
-# Scipy, CFFI, jinja2 and IPython are optional dependencies, but exercised in the test suite
+
 if [ "$PYTHON" == "pypy" ]; then
     conda create -c gmarkall -n $CONDA_ENV -q -y pypy
 else
@@ -27,7 +23,11 @@ source activate $CONDA_ENV
 set -v
 
 # Install llvmdev (separate channel, for now)
-$CONDA_INSTALL -c numba/label/dev llvmdev="14.*"
+if [ "$LLVM" == "16" ]; then
+    $CONDA_INSTALL -c conda-forge llvmdev="16"
+else
+    $CONDA_INSTALL -c numba llvmdev="15.*"
+fi
 
 # Install the compiler toolchain, for osx, bootstrapping needed
 # which happens in build.sh
