@@ -9,10 +9,18 @@ set -v -e
 
 python --version
 
+if [ "$OPAQUE_POINTERS" == "yes" ]; then
+    export LLVMLITE_ENABLE_OPAQUE_POINTERS=1
+    echo "Testing with opaque pointers enabled"
+else
+    echo "Testing with opaque pointers disabled"
+fi
+
 if [ "$WHEEL" == "yes" ]; then
     cd dist
     python -m llvmlite.tests -v
 else
     python runtests.py -v
 fi
+
 if [ "$RUN_COVERAGE" == "yes" ]; then coverage run runtests.py; fi
