@@ -20,11 +20,21 @@ LLVMPY_CreateByteString(const char *buf, size_t len) {
 API_EXPORT(void)
 LLVMPY_DisposeString(const char *msg) { free(const_cast<char *>(msg)); }
 
+// FIXME: Remove `enableOpaquePointers' once typed pointers are removed.
 API_EXPORT(LLVMContextRef)
-LLVMPY_GetGlobalContext() { return LLVMGetGlobalContext(); }
+LLVMPY_GetGlobalContext(bool enableOpaquePointers) {
+    auto context = LLVMGetGlobalContext();
+    LLVMContextSetOpaquePointers(context, enableOpaquePointers);
+    return context;
+}
 
+// FIXME: Remove `enableOpaquePointers' once typed pointers are removed.
 API_EXPORT(LLVMContextRef)
-LLVMPY_ContextCreate() { return LLVMContextCreate(); }
+LLVMPY_ContextCreate(bool enableOpaquePointers) {
+    LLVMContextRef context = LLVMContextCreate();
+    LLVMContextSetOpaquePointers(context, enableOpaquePointers);
+    return context;
+}
 
 API_EXPORT(void)
 LLVMPY_ContextDispose(LLVMContextRef context) {
