@@ -3,7 +3,12 @@ from llvmlite import ir
 from llvmlite import binding as llvm
 from llvmlite.tests import TestCase
 
-from . import refprune_proto as proto
+import llvmlite.tests.refprune_proto as proto
+
+# TODO:: Get rid of Legacy tests once completely transitioned to NewPassManager
+
+# FIXME: Remove me once typed pointers are no longer supported.
+from llvmlite import opaque_pointers_enabled
 
 
 def _iterate_cases(generate_test):
@@ -18,6 +23,15 @@ def _iterate_cases(generate_test):
             yield f'test_{k}', wrap(case_fn)
 
 
+class PassManagerMixin():
+
+    def pb(self):
+        llvm.initialize_native_target()
+        tm = llvm.Target.from_default_triple().create_target_machine()
+        pto = llvm.create_pipeline_tuning_options(speed_level=0, size_level=0)
+        return llvm.create_pass_builder(tm, pto)
+
+
 class TestRefPrunePrototype(TestCase):
     """
     Test that the prototype is working.
@@ -35,7 +49,7 @@ class TestRefPrunePrototype(TestCase):
 ptr_ty = ir.IntType(8).as_pointer()
 
 
-class TestRefPrunePass(TestCase):
+class TestRefPrunePass(TestCase, PassManagerMixin):
     """
     Test that the C++ implementation matches the expected behavior as for
     the prototype.
@@ -115,6 +129,14 @@ class TestRefPrunePass(TestCase):
         return m
 
     def apply_refprune(self, irmod):
+        mod = llvm.parse_assembly(str(irmod))
+        pb = self.pb()
+        pm = pb.getModulePassManager()
+        pm.add_refprune_pass()
+        pm.run(mod, pb)
+        return mod
+
+    def apply_refprune_legacy(self, irmod):
         mod = llvm.parse_assembly(str(irmod))
         pm = llvm.ModulePassManager()
         pm.add_refprune_pass()
@@ -153,12 +175,21 @@ class TestRefPrunePass(TestCase):
         outmod = self.apply_refprune(irmod)
         self.check(outmod, expected, nodes)
 
+    def generate_test_legacy(self, case_gen):
+        nodes, edges, expected = case_gen()
+        irmod = self.generate_ir(nodes, edges)
+        outmod = self.apply_refprune_legacy(irmod)
+        self.check(outmod, expected, nodes)
+
     # Generate tests
     for name, case in _iterate_cases(generate_test):
         locals()[name] = case
 
+    for name, case in _iterate_cases(generate_test_legacy):
+        locals()[name + "_legacy"] = case
 
-class BaseTestByIR(TestCase):
+
+class BaseTestByIR(TestCase, PassManagerMixin):
     refprune_bitmask = 0
 
     prologue = r"""
@@ -167,6 +198,20 @@ declare void @NRT_decref(i8* %ptr)
 """
 
     def check(self, irmod, subgraph_limit=None):
+        mod = llvm.parse_assembly(f"{self.prologue}\n{irmod}")
+        pb = self.pb()
+        pm = pb.getModulePassManager()
+        if subgraph_limit is None:
+            pm.add_refprune_pass(self.refprune_bitmask)
+        else:
+            pm.add_refprune_pass(self.refprune_bitmask,
+                                 subgraph_limit=subgraph_limit)
+        before = llvm.dump_refprune_stats()
+        pm.run(mod, pb)
+        after = llvm.dump_refprune_stats()
+        return mod, after - before
+
+    def check_legacy(self, irmod, subgraph_limit=None):
         mod = llvm.parse_assembly(f"{self.prologue}\n{irmod}")
         pm = llvm.ModulePassManager()
         if subgraph_limit is None:
@@ -195,6 +240,10 @@ define void @main(i8* %ptr) {
         mod, stats = self.check(self.per_bb_ir_1)
         self.assertEqual(stats.basicblock, 2)
 
+    def test_per_bb_1_legacy(self):
+        mod, stats = self.check_legacy(self.per_bb_ir_1)
+        self.assertEqual(stats.basicblock, 2)
+
     per_bb_ir_2 = r"""
 define void @main(i8* %ptr) {
     call void @NRT_incref(i8* %ptr)
@@ -210,9 +259,31 @@ define void @main(i8* %ptr) {
         mod, stats = self.check(self.per_bb_ir_2)
         self.assertEqual(stats.basicblock, 4)
         # not pruned
-        self.assertIn("call void @NRT_incref(i8* %ptr)", str(mod))
+        # FIXME: Remove `else' once TP are no longer supported.
+        if opaque_pointers_enabled:
+            self.assertIn("call void @NRT_incref(ptr %ptr)", str(mod))
+        else:
+            self.assertIn("call void @NRT_incref(i8* %ptr)", str(mod))
+
+    def test_per_bb_2_legacy(self):
+        mod, stats = self.check_legacy(self.per_bb_ir_2)
+        self.assertEqual(stats.basicblock, 4)
+        # not pruned
+        # FIXME: Remove `else' once TP are no longer supported.
+        if opaque_pointers_enabled:
+            self.assertIn("call void @NRT_incref(ptr %ptr)", str(mod))
+        else:
+            self.assertIn("call void @NRT_incref(i8* %ptr)", str(mod))
 
     per_bb_ir_3 = r"""
+define void @main(ptr %ptr, ptr %other) {
+    call void @NRT_incref(ptr %ptr)
+    call void @NRT_incref(ptr %ptr)
+    call void @NRT_decref(ptr %ptr)
+    call void @NRT_decref(ptr %other)
+    ret void
+}
+""" if opaque_pointers_enabled else r"""
 define void @main(i8* %ptr, i8* %other) {
     call void @NRT_incref(i8* %ptr)
     call void @NRT_incref(i8* %ptr)
@@ -226,10 +297,34 @@ define void @main(i8* %ptr, i8* %other) {
         mod, stats = self.check(self.per_bb_ir_3)
         self.assertEqual(stats.basicblock, 2)
         # not pruned
-        self.assertIn("call void @NRT_decref(i8* %other)", str(mod))
+        # FIXME: Remove `else' once TP are no longer supported.
+        if opaque_pointers_enabled:
+            self.assertIn("call void @NRT_decref(ptr %other)", str(mod))
+        else:
+            self.assertIn("call void @NRT_decref(i8* %other)", str(mod))
+
+    def test_per_bb_3_legacy(self):
+        mod, stats = self.check_legacy(self.per_bb_ir_3)
+        self.assertEqual(stats.basicblock, 2)
+        # not pruned
+        # FIXME: Remove `else' once TP are no longer supported.
+        if opaque_pointers_enabled:
+            self.assertIn("call void @NRT_decref(ptr %other)", str(mod))
+        else:
+            self.assertIn("call void @NRT_decref(i8* %other)", str(mod))
 
     per_bb_ir_4 = r"""
 ; reordered
+define void @main(ptr %ptr, ptr %other) {
+    call void @NRT_incref(ptr %ptr)
+    call void @NRT_decref(ptr %ptr)
+    call void @NRT_decref(ptr %ptr)
+    call void @NRT_decref(ptr %other)
+    call void @NRT_incref(ptr %ptr)
+    ret void
+}
+""" if opaque_pointers_enabled else r"""
+; reordered
 define void @main(i8* %ptr, i8* %other) {
     call void @NRT_incref(i8* %ptr)
     call void @NRT_decref(i8* %ptr)
@@ -244,7 +339,21 @@ define void @main(i8* %ptr, i8* %other) {
         mod, stats = self.check(self.per_bb_ir_4)
         self.assertEqual(stats.basicblock, 4)
         # not pruned
-        self.assertIn("call void @NRT_decref(i8* %other)", str(mod))
+        # FIXME: Remove `else' once TP are no longer supported.
+        if opaque_pointers_enabled:
+            self.assertIn("call void @NRT_decref(ptr %other)", str(mod))
+        else:
+            self.assertIn("call void @NRT_decref(i8* %other)", str(mod))
+
+    def test_per_bb_4_legacy(self):
+        mod, stats = self.check_legacy(self.per_bb_ir_4)
+        self.assertEqual(stats.basicblock, 4)
+        # not pruned
+        # FIXME: Remove `else' once TP are no longer supported.
+        if opaque_pointers_enabled:
+            self.assertIn("call void @NRT_decref(ptr %other)", str(mod))
+        else:
+            self.assertIn("call void @NRT_decref(i8* %other)", str(mod))
 
 
 class TestDiamond(BaseTestByIR):
@@ -265,6 +374,10 @@ bb_B:
         mod, stats = self.check(self.per_diamond_1)
         self.assertEqual(stats.diamond, 2)
 
+    def test_per_diamond_1_legacy(self):
+        mod, stats = self.check_legacy(self.per_diamond_1)
+        self.assertEqual(stats.diamond, 2)
+
     per_diamond_2 = r"""
 define void @main(i8* %ptr, i1 %cond) {
 bb_A:
@@ -284,6 +397,10 @@ bb_D:
         mod, stats = self.check(self.per_diamond_2)
         self.assertEqual(stats.diamond, 2)
 
+    def test_per_diamond_2_legacy(self):
+        mod, stats = self.check_legacy(self.per_diamond_2)
+        self.assertEqual(stats.diamond, 2)
+
     per_diamond_3 = r"""
 define void @main(i8* %ptr, i1 %cond) {
 bb_A:
@@ -304,6 +421,10 @@ bb_D:
         mod, stats = self.check(self.per_diamond_3)
         self.assertEqual(stats.diamond, 0)
 
+    def test_per_diamond_3_legacy(self):
+        mod, stats = self.check_legacy(self.per_diamond_3)
+        self.assertEqual(stats.diamond, 0)
+
     per_diamond_4 = r"""
 define void @main(i8* %ptr, i1 %cond) {
 bb_A:
@@ -324,6 +445,10 @@ bb_D:
         mod, stats = self.check(self.per_diamond_4)
         self.assertEqual(stats.diamond, 2)
 
+    def test_per_diamond_4_legacy(self):
+        mod, stats = self.check_legacy(self.per_diamond_4)
+        self.assertEqual(stats.diamond, 2)
+
     per_diamond_5 = r"""
 define void @main(i8* %ptr, i1 %cond) {
 bb_A:
@@ -345,6 +470,10 @@ bb_D:
         mod, stats = self.check(self.per_diamond_5)
         self.assertEqual(stats.diamond, 4)
 
+    def test_per_diamond_5_legacy(self):
+        mod, stats = self.check_legacy(self.per_diamond_5)
+        self.assertEqual(stats.diamond, 4)
+
 
 class TestFanout(BaseTestByIR):
     """More complex cases are tested in TestRefPrunePass
@@ -370,8 +499,12 @@ bb_C:
         mod, stats = self.check(self.fanout_1)
         self.assertEqual(stats.fanout, 3)
 
+    def test_fanout_1_legacy(self):
+        mod, stats = self.check_legacy(self.fanout_1)
+        self.assertEqual(stats.fanout, 3)
+
     fanout_2 = r"""
-define void @main(i8* %ptr, i1 %cond) {
+define void @main(i8* %ptr, i1 %cond, i8** %excinfo) {
 bb_A:
     call void @NRT_incref(i8* %ptr)
     br i1 %cond, label %bb_B, label %bb_C
@@ -388,6 +521,10 @@ bb_C:
         mod, stats = self.check(self.fanout_2)
         self.assertEqual(stats.fanout, 0)
 
+    def test_fanout_2_legacy(self):
+        mod, stats = self.check_legacy(self.fanout_2)
+        self.assertEqual(stats.fanout, 0)
+
     fanout_3 = r"""
 define void @main(i8* %ptr, i1 %cond) {
 bb_A:
@@ -416,12 +553,22 @@ bb_C:
         mod, stats = self.check(self.fanout_3, subgraph_limit=1)
         self.assertEqual(stats.fanout, 0)
 
+    def test_fanout_3_legacy(self):
+        mod, stats = self.check_legacy(self.fanout_3)
+        self.assertEqual(stats.fanout, 6)
+
+    def test_fanout_3_limited_legacy(self):
+        # With subgraph limit at 1, it is essentially turning off the fanout
+        # pruner.
+        mod, stats = self.check_legacy(self.fanout_3, subgraph_limit=1)
+        self.assertEqual(stats.fanout, 0)
+
 
 class TestFanoutRaise(BaseTestByIR):
     refprune_bitmask = llvm.RefPruneSubpasses.FANOUT_RAISE
 
     fanout_raise_1 = r"""
-define i32 @main(i8* %ptr, i1 %cond) {
+define i32 @main(i8* %ptr, i1 %cond, i8** %excinfo) {
 bb_A:
     call void @NRT_incref(i8* %ptr)
     br i1 %cond, label %bb_B, label %bb_C
@@ -429,9 +576,9 @@ bb_B:
     call void @NRT_decref(i8* %ptr)
     ret i32 0
 bb_C:
-    ret i32 1, !ret_is_raise !0
+    store i8* null, i8** %excinfo, !numba_exception_output !0
+    ret i32 1
 }
-
 !0 = !{i1 true}
 """
 
@@ -439,8 +586,12 @@ bb_C:
         mod, stats = self.check(self.fanout_raise_1)
         self.assertEqual(stats.fanout_raise, 2)
 
+    def test_fanout_raise_1_legacy(self):
+        mod, stats = self.check_legacy(self.fanout_raise_1)
+        self.assertEqual(stats.fanout_raise, 2)
+
     fanout_raise_2 = r"""
-define i32 @main(i8* %ptr, i1 %cond) {
+define i32 @main(i8* %ptr, i1 %cond, i8** %excinfo) {
 bb_A:
     call void @NRT_incref(i8* %ptr)
     br i1 %cond, label %bb_B, label %bb_C
@@ -448,18 +599,27 @@ bb_B:
     call void @NRT_decref(i8* %ptr)
     ret i32 0
 bb_C:
-    ret i32 1, !ret_is_a_raise !0           ; bad metadata
+    store i8* null, i8** %excinfo, !numba_exception_typo !0      ; bad metadata
+    ret i32 1
 }
 
 !0 = !{i1 true}
 """
 
     def test_fanout_raise_2(self):
+        # This is ensuring that fanout_raise is not pruning when the metadata
+        # is incorrectly named.
         mod, stats = self.check(self.fanout_raise_2)
         self.assertEqual(stats.fanout_raise, 0)
 
+    def test_fanout_raise_2_legacy(self):
+        # This is ensuring that fanout_raise is not pruning when the metadata
+        # is incorrectly named.
+        mod, stats = self.check_legacy(self.fanout_raise_2)
+        self.assertEqual(stats.fanout_raise, 0)
+
     fanout_raise_3 = r"""
-define i32 @main(i8* %ptr, i1 %cond) {
+define i32 @main(i8* %ptr, i1 %cond, i8** %excinfo) {
 bb_A:
     call void @NRT_incref(i8* %ptr)
     br i1 %cond, label %bb_B, label %bb_C
@@ -467,7 +627,8 @@ bb_B:
     call void @NRT_decref(i8* %ptr)
     ret i32 0
 bb_C:
-    ret i32 1, !ret_is_raise !0
+    store i8* null, i8** %excinfo, !numba_exception_output !0
+    ret i32 1
 }
 
 !0 = !{i32 1}       ; ok; use i32
@@ -477,24 +638,93 @@ bb_C:
         mod, stats = self.check(self.fanout_raise_3)
         self.assertEqual(stats.fanout_raise, 2)
 
+    def test_fanout_raise_3_legacy(self):
+        mod, stats = self.check_legacy(self.fanout_raise_3)
+        self.assertEqual(stats.fanout_raise, 2)
+
     fanout_raise_4 = r"""
-define i32 @main(i8* %ptr, i1 %cond) {
+define i32 @main(i8* %ptr, i1 %cond, i8** %excinfo) {
 bb_A:
     call void @NRT_incref(i8* %ptr)
     br i1 %cond, label %bb_B, label %bb_C
 bb_B:
-    ret i32 1, !ret_is_raise !0    ; BAD; all tails are raising without decref
+    ret i32 1    ; BAD; all tails are raising without decref
 bb_C:
-    ret i32 1, !ret_is_raise !0    ; BAD; all tails are raising without decref
+    ret i32 1    ; BAD; all tails are raising without decref
 }
 
-!0 = !{i32 1}
+!0 = !{i1 1}
 """
 
     def test_fanout_raise_4(self):
         mod, stats = self.check(self.fanout_raise_4)
         self.assertEqual(stats.fanout_raise, 0)
 
+    def test_fanout_raise_4_legacy(self):
+        mod, stats = self.check_legacy(self.fanout_raise_4)
+        self.assertEqual(stats.fanout_raise, 0)
+
+    fanout_raise_5 = r"""
+define i32 @main(i8* %ptr, i1 %cond, i8** %excinfo) {
+bb_A:
+    call void @NRT_incref(i8* %ptr)
+    br i1 %cond, label %bb_B, label %bb_C
+bb_B:
+    call void @NRT_decref(i8* %ptr)
+    br label %common.ret
+bb_C:
+    store i8* null, i8** %excinfo, !numba_exception_output !0
+    br label %common.ret
+common.ret:
+    %common.ret.op = phi i32 [ 0, %bb_B ], [ 1, %bb_C ]
+    ret i32 %common.ret.op
+}
+!0 = !{i1 1}
+"""
+
+    def test_fanout_raise_5(self):
+        mod, stats = self.check(self.fanout_raise_5)
+        self.assertEqual(stats.fanout_raise, 2)
+
+    def test_fanout_raise_5_legacy(self):
+        mod, stats = self.check_legacy(self.fanout_raise_5)
+        self.assertEqual(stats.fanout_raise, 2)
+
+    # test case 6 is from https://github.com/numba/llvmlite/issues/1023
+    fanout_raise_6 = r"""
+define i32 @main(i8* %ptr, i1 %cond1, i1 %cond2, i1 %cond3, i8** %excinfo) {
+bb_A:
+    call void @NRT_incref(i8* %ptr)
+    call void @NRT_incref(i8* %ptr)
+    br i1 %cond1, label %bb_B, label %bb_C
+bb_B:
+    call void @NRT_decref(i8* %ptr)
+    br i1 %cond2, label %bb_D, label %bb_E
+bb_C:
+    store i8* null, i8** %excinfo, !numba_exception_output !0
+    ret i32 1
+bb_D:
+    call void @NRT_decref(i8* %ptr)
+    ret i32 0
+bb_E:
+    call void @NRT_incref(i8* %ptr)
+    br i1 %cond3, label %bb_F, label %bb_C
+bb_F:
+    call void @NRT_decref(i8* %ptr)
+    call void @NRT_decref(i8* %ptr)
+    ret i32 0
+}
+!0 = !{i1 1}
+"""
+
+    def test_fanout_raise_6(self):
+        mod, stats = self.check(self.fanout_raise_6)
+        self.assertEqual(stats.fanout_raise, 7)
+
+    def test_fanout_raise_6_legacy(self):
+        mod, stats = self.check_legacy(self.fanout_raise_6)
+        self.assertEqual(stats.fanout_raise, 7)
+
 
 if __name__ == '__main__':
     unittest.main()
